package com.fuyuanshen.app.service.impl;

import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.fuyuanshen.app.domain.AppUser;
import com.fuyuanshen.app.domain.bo.AppUserBo;
import com.fuyuanshen.app.domain.dto.APPForgotPasswordDTO;
import com.fuyuanshen.app.domain.dto.APPForgotPasswordSmsDTO;
import com.fuyuanshen.app.domain.dto.APPUpdateUserDTO;
import com.fuyuanshen.app.domain.vo.APPUserInfoVo;
import com.fuyuanshen.app.domain.vo.AppUserVo;
import com.fuyuanshen.app.mapper.AppUserMapper;
import com.fuyuanshen.app.service.IAppUserService;
import com.fuyuanshen.common.core.constant.GlobalConstants;
import com.fuyuanshen.common.core.domain.model.AppLoginUser;
import com.fuyuanshen.common.core.exception.BadRequestException;
import com.fuyuanshen.common.core.utils.MapstructUtils;
import com.fuyuanshen.common.core.utils.StringUtils;
import com.fuyuanshen.common.mybatis.core.page.PageQuery;
import com.fuyuanshen.common.mybatis.core.page.TableDataInfo;
import com.fuyuanshen.common.redis.utils.RedisUtils;
import com.fuyuanshen.common.satoken.utils.AppLoginHelper;
import com.fuyuanshen.system.domain.vo.SysOssVo;
import com.fuyuanshen.system.service.ISysOssService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.util.Collection;
import java.util.List;
import java.util.Map;

/**
 * Service业务层处理
 *
 * @author Lion Li
 * @date 2025-06-27
 */
@Slf4j
@RequiredArgsConstructor
@Service
public class AppUserServiceImpl implements IAppUserService {

    private final AppUserMapper baseMapper;

    private final ISysOssService sysOssService;

    /**
     * 查询APP用户信息
     *
     * @param userId 主键
     * @return APP用户信息
     */
    @Override
    public AppUserVo queryById(Long userId){
        return baseMapper.selectVoById(userId);
    }

    /**
     * 分页查询APP用户信息列表
     *
     * @param bo        查询条件
     * @param pageQuery 分页参数
     * @return APP用户信息分页列表
     */
    @Override
    public TableDataInfo<AppUserVo> queryPageList(AppUserBo bo, PageQuery pageQuery) {
        LambdaQueryWrapper<AppUser> lqw = buildQueryWrapper(bo);
        Page<AppUserVo> result = baseMapper.selectVoPage(pageQuery.build(), lqw);
        return TableDataInfo.build(result);
    }

    /**
     * 查询符合条件的APP用户信息列表
     *
     * @param bo 查询条件
     * @return APP用户信息列表
     */
    @Override
    public List<AppUserVo> queryList(AppUserBo bo) {
        LambdaQueryWrapper<AppUser> lqw = buildQueryWrapper(bo);
        return baseMapper.selectVoList(lqw);
    }

    private LambdaQueryWrapper<AppUser> buildQueryWrapper(AppUserBo bo) {
        Map<String, Object> params = bo.getParams();
        LambdaQueryWrapper<AppUser> lqw = Wrappers.lambdaQuery();
        lqw.orderByAsc(AppUser::getUserId);
        lqw.like(StringUtils.isNotBlank(bo.getUserName()), AppUser::getUserName, bo.getUserName());
        lqw.like(StringUtils.isNotBlank(bo.getNickName()), AppUser::getNickName, bo.getNickName());
        lqw.eq(StringUtils.isNotBlank(bo.getUserType()), AppUser::getUserType, bo.getUserType());
        lqw.eq(StringUtils.isNotBlank(bo.getEmail()), AppUser::getEmail, bo.getEmail());
        lqw.eq(StringUtils.isNotBlank(bo.getPhonenumber()), AppUser::getPhonenumber, bo.getPhonenumber());
        lqw.eq(StringUtils.isNotBlank(bo.getSex()), AppUser::getSex, bo.getSex());
        lqw.eq(bo.getAvatar() != null, AppUser::getAvatar, bo.getAvatar());
        lqw.eq(StringUtils.isNotBlank(bo.getPassword()), AppUser::getPassword, bo.getPassword());
        lqw.eq(StringUtils.isNotBlank(bo.getStatus()), AppUser::getStatus, bo.getStatus());
        lqw.eq(StringUtils.isNotBlank(bo.getLoginIp()), AppUser::getLoginIp, bo.getLoginIp());
        lqw.eq(bo.getLoginDate() != null, AppUser::getLoginDate, bo.getLoginDate());
        return lqw;
    }

    /**
     * 新增APP用户信息
     *
     * @param bo APP用户信息
     * @return 是否新增成功
     */
    @Override
    public Boolean insertByBo(AppUserBo bo) {
        AppUser add = MapstructUtils.convert(bo, AppUser.class);
        validEntityBeforeSave(add);
        boolean flag = baseMapper.insert(add) > 0;
        if (flag) {
            bo.setUserId(add.getUserId());
        }
        return flag;
    }

    /**
     * 修改APP用户信息
     *
     * @param bo APP用户信息
     * @return 是否修改成功
     */
    @Override
    public Boolean updateByBo(AppUserBo bo) {
        AppUser update = MapstructUtils.convert(bo, AppUser.class);
        validEntityBeforeSave(update);
        return baseMapper.updateById(update) > 0;
    }

    /**
     * 保存前的数据校验
     */
    private void validEntityBeforeSave(AppUser entity){
        //TODO 做一些数据校验,如唯一约束
    }

    /**
     * 校验并批量删除APP用户信息信息
     *
     * @param ids     待删除的主键集合
     * @param isValid 是否进行有效性校验
     * @return 是否删除成功
     */
    @Override
    public Boolean deleteWithValidByIds(Collection<Long> ids, Boolean isValid) {
        if(isValid){
            //TODO 做一些业务上的校验,判断是否需要校验
        }
        return baseMapper.deleteByIds(ids) > 0;
    }

    @Override
    public APPUserInfoVo getUserInfo() {
        String username = AppLoginHelper.getUsername();
        QueryWrapper<AppUser> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("user_name", username);
        List<AppUser> appUsers = baseMapper.selectList(queryWrapper);
        if(appUsers.isEmpty()){
            throw new BadRequestException("用户不存在");
        }
        AppUser user = appUsers.get(0);
//        AppUserVo user = baseMapper.selectVoById(userId);

        APPUserInfoVo appUserVo = new APPUserInfoVo();
        appUserVo.setId(user.getUserId());
        appUserVo.setNickName(user.getNickName());
        appUserVo.setGender(user.getSex());
        appUserVo.setPhone(user.getPhonenumber());
        appUserVo.setRegion(user.getRegion());
        if(user.getAvatar() != null){
            SysOssVo oss = sysOssService.getById(user.getAvatar());
            if(oss != null){
                appUserVo.setAvatarPath(oss.getUrl());
            }
        }

        return appUserVo;
    }

    @Override
    public int updateUser(APPUpdateUserDTO bo) {
        AppLoginUser appUser = AppLoginHelper.getLoginUser();
        AppUserVo appUserVo = baseMapper.selectVoById(appUser.getUserId());
        if(appUserVo == null){
            throw new BadRequestException("用户不存在");
        }
        AppUser updUser= new AppUser();
        updUser.setUserId(appUser.getUserId());
        updUser.setNickName(bo.getNickName());
        if(bo.getFile() != null){
            SysOssVo oss = sysOssService.upload(bo.getFile());
            updUser.setAvatar(oss.getOssId());
        }

        updUser.setRegion(bo.getRegion());
        updUser.setSex(bo.getGender());
        return baseMapper.update(updUser, new LambdaQueryWrapper<AppUser>().eq(AppUser::getUserId, appUser.getUserId()));
    }

    @Override
    public int forgotPassword(APPForgotPasswordDTO bo) {
        AppUser appUser = baseMapper.appFindByUsername(bo.getPhoneNumber());
        if (appUser == null) {
            throw new BadRequestException("手机号不存在");
        }
        String verificationCode = RedisUtils.getCacheObject(GlobalConstants.APP_FORGOT_PASSWORD_SMS_KEY + bo.getPhoneNumber());
        if (verificationCode == null) {
            throw new BadRequestException("验证码已过期");
        }
        if(!bo.getVerificationCode().equals(verificationCode)){
            throw new BadRequestException("验证码错误");
        }
        appUser.setPassword(bo.getPassword());
        baseMapper.updateById(appUser);
        return 1;
    }

    @Override
    public int sendForgotPasswordSms(APPForgotPasswordSmsDTO dto) {
        return 1;
    }
}
